// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.messageBus.extractor;

import com.google.common.base.Preconditions;
import com.google.gson.Gson;
import com.microsoft.azure.servicebus.IMessage;
import lombok.SneakyThrows;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.notification.provider.azure.models.NotificationEventGridServiceBusRequest;
import org.opengroup.osdu.notification.provider.azure.models.NotificationRecordsChangedData;
import org.opengroup.osdu.notification.provider.azure.messageBus.interfaces.IPullRequestBodyExtractor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

import static java.nio.charset.StandardCharsets.UTF_8;

@Component
@Scope(value = "ThreadScope", proxyMode = ScopedProxyMode.TARGET_CLASS)
@ConditionalOnProperty(value = "azure.eventGridToServiceBus.enabled", havingValue = "true", matchIfMissing = false)
public class EventGridServiceBusRequestBodyExtractor implements IPullRequestBodyExtractor {
    private static final Gson GSON = new Gson();
    private NotificationEventGridServiceBusRequest notificationRequest;
    private NotificationRecordsChangedData notificationRecordsChangedData;
    private IMessage message;

    public void InitializeExtractor(IMessage message) {

        this.message = message;
        this.notificationRequest = extractNotificationRequestFromMessageBody();
    }

    public Map<String, String> extractAttributesFromRequestBody() {

        Map<String, String> attributes = new HashMap<>();
        attributes.put("correlation-id", this.notificationRecordsChangedData.getCorrelationId());
        attributes.put("data-partition-id", this.notificationRecordsChangedData.getDataPartitionId());
        attributes.put("account-id", this.notificationRecordsChangedData.getAccountId());
        return attributes;
    }

    public String extractDataFromRequestBody() {

        return notificationRecordsChangedData.getData().toString();
    }

    @SneakyThrows
    private NotificationEventGridServiceBusRequest extractNotificationRequestFromMessageBody() {

        NotificationEventGridServiceBusRequest notificationRequest = null;
        try {
            String requestBody = new String(message.getMessageBody().getBinaryData().get(0), UTF_8);
            NotificationEventGridServiceBusRequest[] notificationRequestArray = GSON.fromJson(requestBody, NotificationEventGridServiceBusRequest[].class);
            notificationRequest = notificationRequestArray[0];
            extractNotificationData(notificationRequest);
        } catch (Exception e) {
            throw new AppException(HttpStatus.BAD_REQUEST.value(), "Request payload parsing error",
                    "Unable to parse request payload.", "Request contents are null or empty", e);
        }
        return notificationRequest;
    }

    private void extractNotificationData(NotificationEventGridServiceBusRequest notificationRequest) {

        String notifData = notificationRequest.getData().toString();
        NotificationRecordsChangedData notificationRecordsChangedData = GSON.fromJson(notifData, NotificationRecordsChangedData.class);
        verifyNotificationData(notificationRecordsChangedData);
        this.notificationRecordsChangedData = notificationRecordsChangedData;
    }

    private void verifyNotificationData(NotificationRecordsChangedData notificationRecordsChangedData) {

        Preconditions.checkNotNull(notificationRecordsChangedData, "Request payload parsing error");
        Preconditions.checkNotNull(notificationRecordsChangedData.getData(), "Request payload parsing error");
        Preconditions.checkNotNull(notificationRecordsChangedData.getCorrelationId(), "Request payload parsing error");
        Preconditions.checkNotNull(notificationRecordsChangedData.getDataPartitionId(), "Request payload parsing error");
    }
}

